/*
  UDPSendReceiveString:
  This sketch receives UDP message strings, prints them to the serial port
  and sends an "acknowledge" string back to the sender

  A Processing sketch is included at the end of file that can be used to send
  and received messages for testing with a computer.

  created 21 Aug 2010
  by Michael Margolis

  This code is in the public domain.
*/
/*
   Edit by kawabe 2016/08/22
*/

#include <SPI.h>                // needed for Arduino versions later than 0018
#include <Ethernet.h>
#include <EthernetUdp.h>         // UDP library from: bjoern@cs.stanford.edu 12/30/2008
// インストールしたフォルダーの配下にあります。
// \arduino-1.6.10\libraries\Ethernet\src\EthernetUdp.h
#include <Wire.h>

#include <string.h>             // for strcmp

typedef struct {
  int HumidityH ;               // 湿度　整数
  int HumidityL;                // 湿度　小数点以下
  int TemperatureH;             // 温度　整数
  int TemperatureL;             // 温度　小数点以下
} HumidityTemperature_t ;

void GetTempHumidi(HumidityTemperature_t ht) ;

#define   GREEN_LED_PIN 8
byte I2C_AM2321 = 0x5c ;                    // AM2321 I2C Slave address
byte I2C_data[8];                           // AM2321 受信データエリア

// Enter a MAC address and IP address for your controller below.
// The IP address will be dependent on your local network:
byte mac[] = { 0x90, 0xA2, 0xDA, 0x00, 0x51, 0xEB };
// Ethernet シールド基板の裏側にシールが貼ってあります。
IPAddress ip(192, 168, 100, 32);

unsigned int localPort = 45600;             // local port to listen on

// buffers for receiving and sending data
#define SEND_UDP_BUFFER_MAX 16
char packetBuffer[UDP_TX_PACKET_MAX_SIZE];  //buffer to hold incoming packet,
char ReplyBuffer[SEND_UDP_BUFFER_MAX] ;     // a string to send back
String strBuffer ;

// An EthernetUDP instance to let us send and receive packets over UDP
EthernetUDP Udp;

//------------------------------------------------------------
//  Setup
//------------------------------------------------------------
// 1.PIN 割り当て
// 2.Ethernet の準備
// 3.UDP 通信の開始
// 4.シリアル通信の開始 ( for Debug )
void setup() {
  pinMode(GREEN_LED_PIN, OUTPUT);

  Wire.begin();     // I2C 準備

  // start the Ethernet and UDP:
  Ethernet.begin(mac, ip);
  Udp.begin(localPort);

  Serial.begin(9600);
}

//------------------------------------------------------------
//  Main Loop
//------------------------------------------------------------
// 1.UDP によりコマンド(ON)受信
// 2.温湿度読み込み指示
// 3.UDP で温度、湿度データを送信する。
void loop() {
  int i ;
  HumidityTemperature_t ht ;
  String strHumidityH ;
  String strHumidityL;
  String strTemperatureH;
  String strTemperatureL;

  // if there's data available, read a packet
  int packetSize = Udp.parsePacket();
  if (packetSize > 0) {
    Serial.println("-------");
    Serial.print("Received packet of size ");
    Serial.println(packetSize);
    Serial.print("From ");
    IPAddress remote = Udp.remoteIP();
    for (int i = 0; i < 4; i++) {
      Serial.print(remote[i], DEC);
      if (i < 3) {
        Serial.print(".");
      }
    }
    Serial.print(", port ");
    Serial.println(Udp.remotePort());

    // read the packet into packetBufffer
    Udp.read(packetBuffer, UDP_TX_PACKET_MAX_SIZE);
    Serial.print("Contents:");
    Serial.println(packetBuffer);
    if ( strcmp( packetBuffer, "ON" ) == 0 ) {  // "ON" を受信した。
      digitalWrite(GREEN_LED_PIN, HIGH);        // Green LED ON ;
      GetTempHumidi(&ht) ;                      // 温湿度データ取得
      // 温湿度データを文字列に変更
      strHumidityH = String( ht.HumidityH ) ;
      strHumidityL = String( ht.HumidityL ) ;
      strTemperatureH = String( ht.TemperatureH ) ;
      strTemperatureL = String( ht.TemperatureL ) ;
      strBuffer = "H=" + strHumidityH ;
      strBuffer += "." ;
      strBuffer += strHumidityL ;
      strBuffer += " T=" ;
      strBuffer += strTemperatureH ;
      strBuffer += "." ;
      strBuffer += strTemperatureL ;
      // strBuffer ==> ReplyBuffer コピー
      ReplyBuffer[0] = 0 ;
      strBuffer.getBytes( ReplyBuffer, SEND_UDP_BUFFER_MAX) ;
      // 温湿度データを UDP により送信する。
      Udp.beginPacket(Udp.remoteIP(), Udp.remotePort());
      Udp.write(ReplyBuffer);
      Udp.endPacket();
      delay(200) ;                              // LED 点灯確認用
      digitalWrite(GREEN_LED_PIN, LOW);         // Green LED OFF ;
    }
    // Clear Buffer
    if ( packetSize >= UDP_TX_PACKET_MAX_SIZE ) {
      packetSize = UDP_TX_PACKET_MAX_SIZE - 1 ; // 念のため
    }
    for ( i = 0 ; i <= packetSize ; i ++ ) {
      packetBuffer[i] = 0 ;
    }
  }
  delay(10);
}

//------------------------------------------------------------
//　AM2321 より温度、湿度の読み込む
//------------------------------------------------------------
void GetTempHumidi(HumidityTemperature_t* ht)
{
  int i ;
  int Humidity16 ;
  int Temperature16 ;

  // AM2321 アクセス
  Wire.beginTransmission(I2C_AM2321);
  Wire.write(0x00);
  Wire.endTransmission();
  delay(1) ;

  Wire.beginTransmission(I2C_AM2321);
  Wire.write(0x03);   // Function read register
  Wire.write(0x00);   // 0 (Top register)
  Wire.write(0x04);   // Read 4 byte
  Wire.endTransmission();
  delay(2) ;
  Wire.requestFrom(I2C_AM2321, 0x08);
  // 湿度、温度データ取得
  i = 0;
  while (Wire.available()) {
    I2C_data[i++] = Wire.read();
  }

  // Humidity     100.0% ==> 0x03E8
  Humidity16 = (I2C_data[2] << 8) + I2C_data[3];
  ht->HumidityH = Humidity16 / 10;
  Serial.print("Humidity=");
  Serial.print(ht->HumidityH);       // 10進で小数点より上を表示
  Serial.print(".");
  ht->HumidityL = Humidity16 % 10;    // 10 で除算した余り
  Serial.print(ht->HumidityL);       // 10進で小数点以下を表示
  Serial.println("%");

  // Temperature
  // I2C_data[4] = 0xFF ; // minus test  FF83 ==> -125,  FFF5 ==> -11
  // I2C_data[5] = 0xF5 ; // minus test　FF75 ==> -139
  Temperature16 = (I2C_data[4] << 8) + I2C_data[5] ;
  ht->TemperatureH = Temperature16 / 10;
  Serial.print("Temperature=");
  Serial.print(ht->TemperatureH );
  Serial.print(".");
  ht->TemperatureL = abs(Temperature16 % 10); // 小数点以下に符号を付けない
  Serial.print(ht->TemperatureL);
  Serial.println(" C");
}

